#include "font.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <assert.h>

char* img_filename(char c){
   // verifier que c est alphanumérique
   assert(('a' <= c && c <= 'z') 
       || ('A' <= c && c <= 'Z') 
       || ('0' <= c && c <= '9'));

   // construction du nom de fichier
   char* res = malloc(20*sizeof(char));
   res[0] = '\0';
   strcat(res, "lettres/");
   if ('A' <= c && c <= 'Z'){
      res[8] = c;
      res[9] = c;
      res[10] = '\0';
   } else {
      res[8] = c;
      res[9] = '\0';
   }
   strcat(res, ".txt");
   return res;
}



void print_bloc(char** G, int n, int m, int K){
   int J = 0; // position du bloc actuel (multiple de K)
   while(J < m){
      for (int i = 0; i < n; ++i){
         // on affiche le bloc actuel, en faisant attention à ne pas dépasser du tableau
         for (int j = 0; j < K && J+j < m; ++j){
            printf("%c", G[i][J+j]);
         }
         printf("\n");
      }
      J += K;
   }
}



char** grille_pixels(char forme, char pix){
   char* fn = img_filename(forme);
   FILE* f = fopen(fn, "r");
   if (f == NULL){
      perror("Fichiers pour les lettres introuvables: vérifiez que le dossier lettres/ est dans le même dossier que l'exécutable");
      exit(1);
   }

   int n; // pour lire dans le fichier
   char** res = malloc(8*sizeof(char*));
   for (int i = 0; i < 8; ++i){
      res[i] = malloc(8*sizeof(char));

      fscanf(f,"%d", &n);
      bool* bits = bits8(n);
      for (int j = 0; j < 8; ++j){
         if (bits[j]){
            res[i][j] = pix;
         } else {
            res[i][j] = ' ';
         }
      }
      free(bits);
   }

   fclose(f);
   free(fn);
   return res;
}


bool* bits8(unsigned char n){
   bool* res = malloc(8*sizeof(bool));
   for (int i = 0; i < 8; ++i){
      res[i] = (n >> (7-i)) & 1;
   }
   return res;
}



void recopier(char** G, char** tampon, int n, int m, int i, int j){
   for (int k = 0; k < n; ++k){
      for (int l = 0; l < m; ++l){
         G[i + k][j+l] = tampon[k][l];
      }
   }
}


char** generer_grille(char* phrase, char pix){
   int n = strlen(phrase);

   // initialiser grille vide
   char** G = malloc(8 * sizeof(char*));
   for (int i = 0; i < 8; ++i){
      G[i] = malloc(n*8*sizeof(char));
      for (int j = 0; j < n*8; ++j){
         G[i][j] = ' ';
      }
   }

   // itérer sur chaque caractère et tamponner l'image correspondante
   // sur la grille
   for (int i = 0; phrase[i] != '\0'; ++i){
      char forme = phrase[i];
      if (forme != ' ' && forme != '\t'){
         char** tampon = grille_pixels(forme, pix);
         recopier(G, tampon, 8, 8, 0, 8*i);
         free2D(tampon, 8);         
      }
   }

   return G;
}

void free2D(char** G, int n){
   for (int i = 0; i < n; ++i){
      free(G[i]);
   }
   free(G);
}



int main(int argc, char **argv){
   if (argc < 3){
      printf("Usage: %s [caractère] [lettres par ligne]\n", argv[0]);
      exit(1);
   }
   int K = atoi(argv[2]);
   char pix = argv[1][0];

   char* phrase = NULL;
   size_t size = 0;
   printf("Rentrez une phrase: ");
   int n = getline(&phrase, &size, stdin) - 1;
   phrase[n] = '\0'; // remplace le retour à la ligne

   char** G = generer_grille(phrase, pix);
   print_bloc(G, 8, 8*n, 8*K);

   free2D(G, 8);
   free(phrase);
   return 0;
}